"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .referencechunk import ReferenceChunk, ReferenceChunkTypedDict
from .textchunk import TextChunk, TextChunkTypedDict
from mistralai.types import BaseModel
from typing import List, Literal, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


ThinkingTypedDict = TypeAliasType(
    "ThinkingTypedDict", Union[ReferenceChunkTypedDict, TextChunkTypedDict]
)


Thinking = TypeAliasType("Thinking", Union[ReferenceChunk, TextChunk])


ThinkChunkType = Literal["thinking",]


class ThinkChunkTypedDict(TypedDict):
    thinking: List[ThinkingTypedDict]
    closed: NotRequired[bool]
    r"""Whether the thinking chunk is closed or not. Currently only used for prefixing."""
    type: NotRequired[ThinkChunkType]


class ThinkChunk(BaseModel):
    thinking: List[Thinking]

    closed: Optional[bool] = None
    r"""Whether the thinking chunk is closed or not. Currently only used for prefixing."""

    type: Optional[ThinkChunkType] = "thinking"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .messageinputcontentchunks import (
    MessageInputContentChunks,
    MessageInputContentChunksTypedDict,
)
from datetime import datetime
from mistralai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Literal, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


Object = Literal["entry",]


MessageInputEntryType = Literal["message.input",]


MessageInputEntryRole = Literal[
    "assistant",
    "user",
]


MessageInputEntryContentTypedDict = TypeAliasType(
    "MessageInputEntryContentTypedDict",
    Union[str, List[MessageInputContentChunksTypedDict]],
)


MessageInputEntryContent = TypeAliasType(
    "MessageInputEntryContent", Union[str, List[MessageInputContentChunks]]
)


class MessageInputEntryTypedDict(TypedDict):
    r"""Representation of an input message inside the conversation."""

    role: MessageInputEntryRole
    content: MessageInputEntryContentTypedDict
    object: NotRequired[Object]
    type: NotRequired[MessageInputEntryType]
    created_at: NotRequired[datetime]
    completed_at: NotRequired[Nullable[datetime]]
    id: NotRequired[str]
    prefix: NotRequired[bool]


class MessageInputEntry(BaseModel):
    r"""Representation of an input message inside the conversation."""

    role: MessageInputEntryRole

    content: MessageInputEntryContent

    object: Optional[Object] = "entry"

    type: Optional[MessageInputEntryType] = "message.input"

    created_at: Optional[datetime] = None

    completed_at: OptionalNullable[datetime] = UNSET

    id: Optional[str] = None

    prefix: Optional[bool] = False

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "object",
            "type",
            "created_at",
            "completed_at",
            "id",
            "prefix",
        ]
        nullable_fields = ["completed_at"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from mistralai.types import BaseModel
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


AgentHandoffStartedEventType = Literal["agent.handoff.started",]


class AgentHandoffStartedEventTypedDict(TypedDict):
    id: str
    previous_agent_id: str
    previous_agent_name: str
    type: NotRequired[AgentHandoffStartedEventType]
    created_at: NotRequired[datetime]
    output_index: NotRequired[int]


class AgentHandoffStartedEvent(BaseModel):
    id: str

    previous_agent_id: str

    previous_agent_name: str

    type: Optional[AgentHandoffStartedEventType] = "agent.handoff.started"

    created_at: Optional[datetime] = None

    output_index: Optional[int] = 0



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from mistralai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class LibraryOutTypedDict(TypedDict):
    id: str
    name: str
    created_at: datetime
    updated_at: datetime
    owner_id: Nullable[str]
    owner_type: str
    total_size: int
    nb_documents: int
    chunk_size: Nullable[int]
    emoji: NotRequired[Nullable[str]]
    description: NotRequired[Nullable[str]]
    generated_description: NotRequired[Nullable[str]]
    explicit_user_members_count: NotRequired[Nullable[int]]
    explicit_workspace_members_count: NotRequired[Nullable[int]]
    org_sharing_role: NotRequired[Nullable[str]]
    generated_name: NotRequired[Nullable[str]]
    r"""Generated Name"""


class LibraryOut(BaseModel):
    id: str

    name: str

    created_at: datetime

    updated_at: datetime

    owner_id: Nullable[str]

    owner_type: str

    total_size: int

    nb_documents: int

    chunk_size: Nullable[int]

    emoji: OptionalNullable[str] = UNSET

    description: OptionalNullable[str] = UNSET

    generated_description: OptionalNullable[str] = UNSET

    explicit_user_members_count: OptionalNullable[int] = UNSET

    explicit_workspace_members_count: OptionalNullable[int] = UNSET

    org_sharing_role: OptionalNullable[str] = UNSET

    generated_name: OptionalNullable[str] = UNSET
    r"""Generated Name"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "emoji",
            "description",
            "generated_description",
            "explicit_user_members_count",
            "explicit_workspace_members_count",
            "org_sharing_role",
            "generated_name",
        ]
        nullable_fields = [
            "owner_id",
            "chunk_size",
            "emoji",
            "description",
            "generated_description",
            "explicit_user_members_count",
            "explicit_workspace_members_count",
            "org_sharing_role",
            "generated_name",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .classifierdetailedjobout import (
    ClassifierDetailedJobOut,
    ClassifierDetailedJobOutTypedDict,
)
from .completiondetailedjobout import (
    CompletionDetailedJobOut,
    CompletionDetailedJobOutTypedDict,
)
from mistralai.types import BaseModel
from mistralai.utils import FieldMetadata, PathParamMetadata, get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


class JobsAPIRoutesFineTuningCancelFineTuningJobRequestTypedDict(TypedDict):
    job_id: str
    r"""The ID of the job to cancel."""


class JobsAPIRoutesFineTuningCancelFineTuningJobRequest(BaseModel):
    job_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The ID of the job to cancel."""


JobsAPIRoutesFineTuningCancelFineTuningJobResponseTypedDict = TypeAliasType(
    "JobsAPIRoutesFineTuningCancelFineTuningJobResponseTypedDict",
    Union[CompletionDetailedJobOutTypedDict, ClassifierDetailedJobOutTypedDict],
)
r"""OK"""


JobsAPIRoutesFineTuningCancelFineTuningJobResponse = Annotated[
    Union[
        Annotated[ClassifierDetailedJobOut, Tag("classifier")],
        Annotated[CompletionDetailedJobOut, Tag("completion")],
    ],
    Discriminator(lambda m: get_discriminator(m, "job_type", "job_type")),
]
r"""OK"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .batchjobout import BatchJobOut, BatchJobOutTypedDict
from mistralai.types import BaseModel
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


BatchJobsOutObject = Literal["list",]


class BatchJobsOutTypedDict(TypedDict):
    total: int
    data: NotRequired[List[BatchJobOutTypedDict]]
    object: NotRequired[BatchJobsOutObject]


class BatchJobsOut(BaseModel):
    total: int

    data: Optional[List[BatchJobOut]] = None

    object: Optional[BatchJobsOutObject] = "list"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from mistralai_azure.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class OCRImageObjectTypedDict(TypedDict):
    id: str
    r"""Image ID for extracted image in a page"""
    top_left_x: Nullable[int]
    r"""X coordinate of top-left corner of the extracted image"""
    top_left_y: Nullable[int]
    r"""Y coordinate of top-left corner of the extracted image"""
    bottom_right_x: Nullable[int]
    r"""X coordinate of bottom-right corner of the extracted image"""
    bottom_right_y: Nullable[int]
    r"""Y coordinate of bottom-right corner of the extracted image"""
    image_base64: NotRequired[Nullable[str]]
    r"""Base64 string of the extracted image"""
    image_annotation: NotRequired[Nullable[str]]
    r"""Annotation of the extracted image in json str"""


class OCRImageObject(BaseModel):
    id: str
    r"""Image ID for extracted image in a page"""

    top_left_x: Nullable[int]
    r"""X coordinate of top-left corner of the extracted image"""

    top_left_y: Nullable[int]
    r"""Y coordinate of top-left corner of the extracted image"""

    bottom_right_x: Nullable[int]
    r"""X coordinate of bottom-right corner of the extracted image"""

    bottom_right_y: Nullable[int]
    r"""Y coordinate of bottom-right corner of the extracted image"""

    image_base64: OptionalNullable[str] = UNSET
    r"""Base64 string of the extracted image"""

    image_annotation: OptionalNullable[str] = UNSET
    r"""Annotation of the extracted image in json str"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["image_base64", "image_annotation"]
        nullable_fields = [
            "top_left_x",
            "top_left_y",
            "bottom_right_x",
            "bottom_right_y",
            "image_base64",
            "image_annotation",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from mistralai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Any, Dict
from typing_extensions import NotRequired, TypedDict


class DocumentOutTypedDict(TypedDict):
    id: str
    library_id: str
    hash: Nullable[str]
    mime_type: Nullable[str]
    extension: Nullable[str]
    size: Nullable[int]
    name: str
    created_at: datetime
    processing_status: str
    uploaded_by_id: Nullable[str]
    uploaded_by_type: str
    tokens_processing_total: int
    summary: NotRequired[Nullable[str]]
    last_processed_at: NotRequired[Nullable[datetime]]
    number_of_pages: NotRequired[Nullable[int]]
    tokens_processing_main_content: NotRequired[Nullable[int]]
    tokens_processing_summary: NotRequired[Nullable[int]]
    url: NotRequired[Nullable[str]]
    attributes: NotRequired[Nullable[Dict[str, Any]]]


class DocumentOut(BaseModel):
    id: str

    library_id: str

    hash: Nullable[str]

    mime_type: Nullable[str]

    extension: Nullable[str]

    size: Nullable[int]

    name: str

    created_at: datetime

    processing_status: str

    uploaded_by_id: Nullable[str]

    uploaded_by_type: str

    tokens_processing_total: int

    summary: OptionalNullable[str] = UNSET

    last_processed_at: OptionalNullable[datetime] = UNSET

    number_of_pages: OptionalNullable[int] = UNSET

    tokens_processing_main_content: OptionalNullable[int] = UNSET

    tokens_processing_summary: OptionalNullable[int] = UNSET

    url: OptionalNullable[str] = UNSET

    attributes: OptionalNullable[Dict[str, Any]] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "summary",
            "last_processed_at",
            "number_of_pages",
            "tokens_processing_main_content",
            "tokens_processing_summary",
            "url",
            "attributes",
        ]
        nullable_fields = [
            "hash",
            "mime_type",
            "extension",
            "size",
            "summary",
            "last_processed_at",
            "number_of_pages",
            "uploaded_by_id",
            "tokens_processing_main_content",
            "tokens_processing_summary",
            "url",
            "attributes",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .file import File, FileTypedDict
from .filepurpose import FilePurpose
from mistralai.types import BaseModel
from mistralai.utils import FieldMetadata, MultipartFormMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class FilesAPIRoutesUploadFileMultiPartBodyParamsTypedDict(TypedDict):
    file: FileTypedDict
    r"""The File object (not file name) to be uploaded.
    To upload a file and specify a custom file name you should format your request as such:
    ```bash
    file=@path/to/your/file.jsonl;filename=custom_name.jsonl
    ```
    Otherwise, you can just keep the original file name:
    ```bash
    file=@path/to/your/file.jsonl
    ```
    """
    purpose: NotRequired[FilePurpose]


class FilesAPIRoutesUploadFileMultiPartBodyParams(BaseModel):
    file: Annotated[File, FieldMetadata(multipart=MultipartFormMetadata(file=True))]
    r"""The File object (not file name) to be uploaded.
    To upload a file and specify a custom file name you should format your request as such:
    ```bash
    file=@path/to/your/file.jsonl;filename=custom_name.jsonl
    ```
    Otherwise, you can just keep the original file name:
    ```bash
    file=@path/to/your/file.jsonl
    ```
    """

    purpose: Annotated[Optional[FilePurpose], FieldMetadata(multipart=True)] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from mistralai.types import BaseModel
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


WebSearchToolType = Literal["web_search",]


class WebSearchToolTypedDict(TypedDict):
    type: NotRequired[WebSearchToolType]


class WebSearchTool(BaseModel):
    type: Optional[WebSearchToolType] = "web_search"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from mistralai.types import BaseModel
from typing_extensions import TypedDict


class ProcessingStatusOutTypedDict(TypedDict):
    document_id: str
    processing_status: str


class ProcessingStatusOut(BaseModel):
    document_id: str

    processing_status: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .documentout import DocumentOut, DocumentOutTypedDict
from .paginationinfo import PaginationInfo, PaginationInfoTypedDict
from mistralai.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListDocumentOutTypedDict(TypedDict):
    pagination: PaginationInfoTypedDict
    data: List[DocumentOutTypedDict]


class ListDocumentOut(BaseModel):
    pagination: PaginationInfo

    data: List[DocumentOut]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .documenturlchunk import DocumentURLChunk, DocumentURLChunkTypedDict
from .filechunk import FileChunk, FileChunkTypedDict
from .imageurlchunk import ImageURLChunk, ImageURLChunkTypedDict
from .responseformat import ResponseFormat, ResponseFormatTypedDict
from mistralai_azure.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import List, Literal, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


DocumentTypedDict = TypeAliasType(
    "DocumentTypedDict",
    Union[FileChunkTypedDict, ImageURLChunkTypedDict, DocumentURLChunkTypedDict],
)
r"""Document to run OCR on"""


Document = TypeAliasType("Document", Union[FileChunk, ImageURLChunk, DocumentURLChunk])
r"""Document to run OCR on"""


TableFormat = Literal[
    "markdown",
    "html",
]


class OCRRequestTypedDict(TypedDict):
    model: Nullable[str]
    document: DocumentTypedDict
    r"""Document to run OCR on"""
    id: NotRequired[str]
    pages: NotRequired[Nullable[List[int]]]
    r"""Specific pages user wants to process in various formats: single number, range, or list of both. Starts from 0"""
    include_image_base64: NotRequired[Nullable[bool]]
    r"""Include image URLs in response"""
    image_limit: NotRequired[Nullable[int]]
    r"""Max images to extract"""
    image_min_size: NotRequired[Nullable[int]]
    r"""Minimum height and width of image to extract"""
    bbox_annotation_format: NotRequired[Nullable[ResponseFormatTypedDict]]
    r"""Structured output class for extracting useful information from each extracted bounding box / image from document. Only json_schema is valid for this field"""
    document_annotation_format: NotRequired[Nullable[ResponseFormatTypedDict]]
    r"""Structured output class for extracting useful information from the entire document. Only json_schema is valid for this field"""
    table_format: NotRequired[Nullable[TableFormat]]
    extract_header: NotRequired[bool]
    extract_footer: NotRequired[bool]


class OCRRequest(BaseModel):
    model: Nullable[str]

    document: Document
    r"""Document to run OCR on"""

    id: Optional[str] = None

    pages: OptionalNullable[List[int]] = UNSET
    r"""Specific pages user wants to process in various formats: single number, range, or list of both. Starts from 0"""

    include_image_base64: OptionalNullable[bool] = UNSET
    r"""Include image URLs in response"""

    image_limit: OptionalNullable[int] = UNSET
    r"""Max images to extract"""

    image_min_size: OptionalNullable[int] = UNSET
    r"""Minimum height and width of image to extract"""

    bbox_annotation_format: OptionalNullable[ResponseFormat] = UNSET
    r"""Structured output class for extracting useful information from each extracted bounding box / image from document. Only json_schema is valid for this field"""

    document_annotation_format: OptionalNullable[ResponseFormat] = UNSET
    r"""Structured output class for extracting useful information from the entire document. Only json_schema is valid for this field"""

    table_format: OptionalNullable[TableFormat] = UNSET

    extract_header: Optional[bool] = None

    extract_footer: Optional[bool] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "pages",
            "include_image_base64",
            "image_limit",
            "image_min_size",
            "bbox_annotation_format",
            "document_annotation_format",
            "table_format",
            "extract_header",
            "extract_footer",
        ]
        nullable_fields = [
            "model",
            "pages",
            "include_image_base64",
            "image_limit",
            "image_min_size",
            "bbox_annotation_format",
            "document_annotation_format",
            "table_format",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from mistralai_azure.types import BaseModel
import pydantic
from typing import Literal
from typing_extensions import Annotated, TypedDict


Format = Literal[
    "markdown",
    "html",
]
r"""Format of the table"""


class OCRTableObjectTypedDict(TypedDict):
    id: str
    r"""Table ID for extracted table in a page"""
    content: str
    r"""Content of the table in the given format"""
    format_: Format
    r"""Format of the table"""


class OCRTableObject(BaseModel):
    id: str
    r"""Table ID for extracted table in a page"""

    content: str
    r"""Content of the table in the given format"""

    format_: Annotated[Format, pydantic.Field(alias="format")]
    r"""Format of the table"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .contentchunk import ContentChunk, ContentChunkTypedDict
from mistralai.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Literal, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


UserMessageContentTypedDict = TypeAliasType(
    "UserMessageContentTypedDict", Union[str, List[ContentChunkTypedDict]]
)


UserMessageContent = TypeAliasType("UserMessageContent", Union[str, List[ContentChunk]])


UserMessageRole = Literal["user",]


class UserMessageTypedDict(TypedDict):
    content: Nullable[UserMessageContentTypedDict]
    role: NotRequired[UserMessageRole]


class UserMessage(BaseModel):
    content: Nullable[UserMessageContent]

    role: Optional[UserMessageRole] = "user"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["role"]
        nullable_fields = ["content"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from mistralai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


AgentHandoffEntryObject = Literal["entry",]


AgentHandoffEntryType = Literal["agent.handoff",]


class AgentHandoffEntryTypedDict(TypedDict):
    previous_agent_id: str
    previous_agent_name: str
    next_agent_id: str
    next_agent_name: str
    object: NotRequired[AgentHandoffEntryObject]
    type: NotRequired[AgentHandoffEntryType]
    created_at: NotRequired[datetime]
    completed_at: NotRequired[Nullable[datetime]]
    id: NotRequired[str]


class AgentHandoffEntry(BaseModel):
    previous_agent_id: str

    previous_agent_name: str

    next_agent_id: str

    next_agent_name: str

    object: Optional[AgentHandoffEntryObject] = "entry"

    type: Optional[AgentHandoffEntryType] = "agent.handoff"

    created_at: Optional[datetime] = None

    completed_at: OptionalNullable[datetime] = UNSET

    id: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["object", "type", "created_at", "completed_at", "id"]
        nullable_fields = ["completed_at"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .systemmessagecontentchunks import (
    SystemMessageContentChunks,
    SystemMessageContentChunksTypedDict,
)
from mistralai.types import BaseModel
from typing import List, Literal, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


SystemMessageContentTypedDict = TypeAliasType(
    "SystemMessageContentTypedDict",
    Union[str, List[SystemMessageContentChunksTypedDict]],
)


SystemMessageContent = TypeAliasType(
    "SystemMessageContent", Union[str, List[SystemMessageContentChunks]]
)


Role = Literal["system",]


class SystemMessageTypedDict(TypedDict):
    content: SystemMessageContentTypedDict
    role: NotRequired[Role]


class SystemMessage(BaseModel):
    content: SystemMessageContent

    role: Optional[Role] = "system"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from mistralai_azure.types import BaseModel
from mistralai_azure.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, TypedDict


class FileChunkTypedDict(TypedDict):
    file_id: str
    type: Literal["file"]


class FileChunk(BaseModel):
    file_id: str

    TYPE: Annotated[
        Annotated[Optional[Literal["file"]], AfterValidator(validate_const("file"))],
        pydantic.Field(alias="type"),
    ] = "file"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .transcriptionstreamdone import (
    TranscriptionStreamDone,
    TranscriptionStreamDoneTypedDict,
)
from .transcriptionstreameventtypes import TranscriptionStreamEventTypes
from .transcriptionstreamlanguage import (
    TranscriptionStreamLanguage,
    TranscriptionStreamLanguageTypedDict,
)
from .transcriptionstreamsegmentdelta import (
    TranscriptionStreamSegmentDelta,
    TranscriptionStreamSegmentDeltaTypedDict,
)
from .transcriptionstreamtextdelta import (
    TranscriptionStreamTextDelta,
    TranscriptionStreamTextDeltaTypedDict,
)
from mistralai.types import BaseModel
from mistralai.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


TranscriptionStreamEventsDataTypedDict = TypeAliasType(
    "TranscriptionStreamEventsDataTypedDict",
    Union[
        TranscriptionStreamTextDeltaTypedDict,
        TranscriptionStreamLanguageTypedDict,
        TranscriptionStreamSegmentDeltaTypedDict,
        TranscriptionStreamDoneTypedDict,
    ],
)


TranscriptionStreamEventsData = Annotated[
    Union[
        Annotated[TranscriptionStreamDone, Tag("transcription.done")],
        Annotated[TranscriptionStreamLanguage, Tag("transcription.language")],
        Annotated[TranscriptionStreamSegmentDelta, Tag("transcription.segment")],
        Annotated[TranscriptionStreamTextDelta, Tag("transcription.text.delta")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


class TranscriptionStreamEventsTypedDict(TypedDict):
    event: TranscriptionStreamEventTypes
    data: TranscriptionStreamEventsDataTypedDict


class TranscriptionStreamEvents(BaseModel):
    event: TranscriptionStreamEventTypes

    data: TranscriptionStreamEventsData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .codeinterpretertool import CodeInterpreterTool, CodeInterpreterToolTypedDict
from .completionargs import CompletionArgs, CompletionArgsTypedDict
from .documentlibrarytool import DocumentLibraryTool, DocumentLibraryToolTypedDict
from .functiontool import FunctionTool, FunctionToolTypedDict
from .imagegenerationtool import ImageGenerationTool, ImageGenerationToolTypedDict
from .websearchpremiumtool import WebSearchPremiumTool, WebSearchPremiumToolTypedDict
from .websearchtool import WebSearchTool, WebSearchToolTypedDict
from datetime import datetime
from mistralai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from mistralai.utils import get_discriminator
from pydantic import Discriminator, Tag, model_serializer
from typing import Any, Dict, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


AgentToolsTypedDict = TypeAliasType(
    "AgentToolsTypedDict",
    Union[
        WebSearchToolTypedDict,
        WebSearchPremiumToolTypedDict,
        CodeInterpreterToolTypedDict,
        ImageGenerationToolTypedDict,
        FunctionToolTypedDict,
        DocumentLibraryToolTypedDict,
    ],
)


AgentTools = Annotated[
    Union[
        Annotated[CodeInterpreterTool, Tag("code_interpreter")],
        Annotated[DocumentLibraryTool, Tag("document_library")],
        Annotated[FunctionTool, Tag("function")],
        Annotated[ImageGenerationTool, Tag("image_generation")],
        Annotated[WebSearchTool, Tag("web_search")],
        Annotated[WebSearchPremiumTool, Tag("web_search_premium")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


AgentObject = Literal["agent",]


class AgentTypedDict(TypedDict):
    model: str
    name: str
    id: str
    version: int
    versions: List[int]
    created_at: datetime
    updated_at: datetime
    deployment_chat: bool
    source: str
    instructions: NotRequired[Nullable[str]]
    r"""Instruction prompt the model will follow during the conversation."""
    tools: NotRequired[List[AgentToolsTypedDict]]
    r"""List of tools which are available to the model during the conversation."""
    completion_args: NotRequired[CompletionArgsTypedDict]
    r"""White-listed arguments from the completion API"""
    description: NotRequired[Nullable[str]]
    handoffs: NotRequired[Nullable[List[str]]]
    metadata: NotRequired[Nullable[Dict[str, Any]]]
    object: NotRequired[AgentObject]


class Agent(BaseModel):
    model: str

    name: str

    id: str

    version: int

    versions: List[int]

    created_at: datetime

    updated_at: datetime

    deployment_chat: bool

    source: str

    instructions: OptionalNullable[str] = UNSET
    r"""Instruction prompt the model will follow during the conversation."""

    tools: Optional[List[AgentTools]] = None
    r"""List of tools which are available to the model during the conversation."""

    completion_args: Optional[CompletionArgs] = None
    r"""White-listed arguments from the completion API"""

    description: OptionalNullable[str] = UNSET

    handoffs: OptionalNullable[List[str]] = UNSET

    metadata: OptionalNullable[Dict[str, Any]] = UNSET

    object: Optional[AgentObject] = "agent"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "instructions",
            "tools",
            "completion_args",
            "description",
            "handoffs",
            "metadata",
            "object",
        ]
        nullable_fields = ["instructions", "description", "handoffs", "metadata"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .function import Function, FunctionTypedDict
from mistralai.types import BaseModel
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


FunctionToolType = Literal["function",]


class FunctionToolTypedDict(TypedDict):
    function: FunctionTypedDict
    type: NotRequired[FunctionToolType]


class FunctionTool(BaseModel):
    function: Function

    type: Optional[FunctionToolType] = "function"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .documenturlchunk import DocumentURLChunk, DocumentURLChunkTypedDict
from .filechunk import FileChunk, FileChunkTypedDict
from .imageurlchunk import ImageURLChunk, ImageURLChunkTypedDict
from .responseformat import ResponseFormat, ResponseFormatTypedDict
from mistralai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Literal, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


DocumentTypedDict = TypeAliasType(
    "DocumentTypedDict",
    Union[FileChunkTypedDict, ImageURLChunkTypedDict, DocumentURLChunkTypedDict],
)
r"""Document to run OCR on"""


Document = TypeAliasType("Document", Union[FileChunk, ImageURLChunk, DocumentURLChunk])
r"""Document to run OCR on"""


TableFormat = Literal[
    "markdown",
    "html",
]


class OCRRequestTypedDict(TypedDict):
    model: Nullable[str]
    document: DocumentTypedDict
    r"""Document to run OCR on"""
    id: NotRequired[str]
    pages: NotRequired[Nullable[List[int]]]
    r"""Specific pages user wants to process in various formats: single number, range, or list of both. Starts from 0"""
    include_image_base64: NotRequired[Nullable[bool]]
    r"""Include image URLs in response"""
    image_limit: NotRequired[Nullable[int]]
    r"""Max images to extract"""
    image_min_size: NotRequired[Nullable[int]]
    r"""Minimum height and width of image to extract"""
    bbox_annotation_format: NotRequired[Nullable[ResponseFormatTypedDict]]
    r"""Structured output class for extracting useful information from each extracted bounding box / image from document. Only json_schema is valid for this field"""
    document_annotation_format: NotRequired[Nullable[ResponseFormatTypedDict]]
    r"""Structured output class for extracting useful information from the entire document. Only json_schema is valid for this field"""
    document_annotation_prompt: NotRequired[Nullable[str]]
    r"""Optional prompt to guide the model in extracting structured output from the entire document. A document_annotation_format must be provided."""
    table_format: NotRequired[Nullable[TableFormat]]
    extract_header: NotRequired[bool]
    extract_footer: NotRequired[bool]


class OCRRequest(BaseModel):
    model: Nullable[str]

    document: Document
    r"""Document to run OCR on"""

    id: Optional[str] = None

    pages: OptionalNullable[List[int]] = UNSET
    r"""Specific pages user wants to process in various formats: single number, range, or list of both. Starts from 0"""

    include_image_base64: OptionalNullable[bool] = UNSET
    r"""Include image URLs in response"""

    image_limit: OptionalNullable[int] = UNSET
    r"""Max images to extract"""

    image_min_size: OptionalNullable[int] = UNSET
    r"""Minimum height and width of image to extract"""

    bbox_annotation_format: OptionalNullable[ResponseFormat] = UNSET
    r"""Structured output class for extracting useful information from each extracted bounding box / image from document. Only json_schema is valid for this field"""

    document_annotation_format: OptionalNullable[ResponseFormat] = UNSET
    r"""Structured output class for extracting useful information from the entire document. Only json_schema is valid for this field"""

    document_annotation_prompt: OptionalNullable[str] = UNSET
    r"""Optional prompt to guide the model in extracting structured output from the entire document. A document_annotation_format must be provided."""

    table_format: OptionalNullable[TableFormat] = UNSET

    extract_header: Optional[bool] = None

    extract_footer: Optional[bool] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "pages",
            "include_image_base64",
            "image_limit",
            "image_min_size",
            "bbox_annotation_format",
            "document_annotation_format",
            "document_annotation_prompt",
            "table_format",
            "extract_header",
            "extract_footer",
        ]
        nullable_fields = [
            "model",
            "pages",
            "include_image_base64",
            "image_limit",
            "image_min_size",
            "bbox_annotation_format",
            "document_annotation_format",
            "document_annotation_prompt",
            "table_format",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .builtinconnectors import BuiltInConnectors
from datetime import datetime
from mistralai.types import BaseModel
from typing import Any, Dict, Literal, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


ToolExecutionDoneEventType = Literal["tool.execution.done",]


ToolExecutionDoneEventNameTypedDict = TypeAliasType(
    "ToolExecutionDoneEventNameTypedDict", Union[BuiltInConnectors, str]
)


ToolExecutionDoneEventName = TypeAliasType(
    "ToolExecutionDoneEventName", Union[BuiltInConnectors, str]
)


class ToolExecutionDoneEventTypedDict(TypedDict):
    id: str
    name: ToolExecutionDoneEventNameTypedDict
    type: NotRequired[ToolExecutionDoneEventType]
    created_at: NotRequired[datetime]
    output_index: NotRequired[int]
    info: NotRequired[Dict[str, Any]]


class ToolExecutionDoneEvent(BaseModel):
    id: str

    name: ToolExecutionDoneEventName

    type: Optional[ToolExecutionDoneEventType] = "tool.execution.done"

    created_at: Optional[datetime] = None

    output_index: Optional[int] = 0

    info: Optional[Dict[str, Any]] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from mistralai.types import BaseModel
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


ResponseStartedEventType = Literal["conversation.response.started",]


class ResponseStartedEventTypedDict(TypedDict):
    conversation_id: str
    type: NotRequired[ResponseStartedEventType]
    created_at: NotRequired[datetime]


class ResponseStartedEvent(BaseModel):
    conversation_id: str

    type: Optional[ResponseStartedEventType] = "conversation.response.started"

    created_at: Optional[datetime] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .classifierftmodelout import ClassifierFTModelOut, ClassifierFTModelOutTypedDict
from .completionftmodelout import CompletionFTModelOut, CompletionFTModelOutTypedDict
from .updateftmodelin import UpdateFTModelIn, UpdateFTModelInTypedDict
from mistralai.types import BaseModel
from mistralai.utils import (
    FieldMetadata,
    PathParamMetadata,
    RequestMetadata,
    get_discriminator,
)
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


class JobsAPIRoutesFineTuningUpdateFineTunedModelRequestTypedDict(TypedDict):
    model_id: str
    r"""The ID of the model to update."""
    update_ft_model_in: UpdateFTModelInTypedDict


class JobsAPIRoutesFineTuningUpdateFineTunedModelRequest(BaseModel):
    model_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The ID of the model to update."""

    update_ft_model_in: Annotated[
        UpdateFTModelIn,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


JobsAPIRoutesFineTuningUpdateFineTunedModelResponseTypedDict = TypeAliasType(
    "JobsAPIRoutesFineTuningUpdateFineTunedModelResponseTypedDict",
    Union[CompletionFTModelOutTypedDict, ClassifierFTModelOutTypedDict],
)
r"""OK"""


JobsAPIRoutesFineTuningUpdateFineTunedModelResponse = Annotated[
    Union[
        Annotated[ClassifierFTModelOut, Tag("classifier")],
        Annotated[CompletionFTModelOut, Tag("completion")],
    ],
    Discriminator(lambda m: get_discriminator(m, "model_type", "model_type")),
]
r"""OK"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .imageurlchunk import ImageURLChunk, ImageURLChunkTypedDict
from .referencechunk import ReferenceChunk, ReferenceChunkTypedDict
from .textchunk import TextChunk, TextChunkTypedDict
from mistralai_gcp.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


ContentChunkTypedDict = TypeAliasType(
    "ContentChunkTypedDict",
    Union[TextChunkTypedDict, ImageURLChunkTypedDict, ReferenceChunkTypedDict],
)


ContentChunk = Annotated[
    Union[
        Annotated[ImageURLChunk, Tag("image_url")],
        Annotated[TextChunk, Tag("text")],
        Annotated[ReferenceChunk, Tag("reference")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .agenthandoffdoneevent import AgentHandoffDoneEvent, AgentHandoffDoneEventTypedDict
from .agenthandoffstartedevent import (
    AgentHandoffStartedEvent,
    AgentHandoffStartedEventTypedDict,
)
from .functioncallevent import FunctionCallEvent, FunctionCallEventTypedDict
from .messageoutputevent import MessageOutputEvent, MessageOutputEventTypedDict
from .responsedoneevent import ResponseDoneEvent, ResponseDoneEventTypedDict
from .responseerrorevent import ResponseErrorEvent, ResponseErrorEventTypedDict
from .responsestartedevent import ResponseStartedEvent, ResponseStartedEventTypedDict
from .ssetypes import SSETypes
from .toolexecutiondeltaevent import (
    ToolExecutionDeltaEvent,
    ToolExecutionDeltaEventTypedDict,
)
from .toolexecutiondoneevent import (
    ToolExecutionDoneEvent,
    ToolExecutionDoneEventTypedDict,
)
from .toolexecutionstartedevent import (
    ToolExecutionStartedEvent,
    ToolExecutionStartedEventTypedDict,
)
from mistralai.types import BaseModel
from mistralai.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


ConversationEventsDataTypedDict = TypeAliasType(
    "ConversationEventsDataTypedDict",
    Union[
        ResponseStartedEventTypedDict,
        ResponseDoneEventTypedDict,
        ResponseErrorEventTypedDict,
        ToolExecutionStartedEventTypedDict,
        ToolExecutionDeltaEventTypedDict,
        ToolExecutionDoneEventTypedDict,
        AgentHandoffStartedEventTypedDict,
        AgentHandoffDoneEventTypedDict,
        FunctionCallEventTypedDict,
        MessageOutputEventTypedDict,
    ],
)


ConversationEventsData = Annotated[
    Union[
        Annotated[AgentHandoffDoneEvent, Tag("agent.handoff.done")],
        Annotated[AgentHandoffStartedEvent, Tag("agent.handoff.started")],
        Annotated[ResponseDoneEvent, Tag("conversation.response.done")],
        Annotated[ResponseErrorEvent, Tag("conversation.response.error")],
        Annotated[ResponseStartedEvent, Tag("conversation.response.started")],
        Annotated[FunctionCallEvent, Tag("function.call.delta")],
        Annotated[MessageOutputEvent, Tag("message.output.delta")],
        Annotated[ToolExecutionDeltaEvent, Tag("tool.execution.delta")],
        Annotated[ToolExecutionDoneEvent, Tag("tool.execution.done")],
        Annotated[ToolExecutionStartedEvent, Tag("tool.execution.started")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


class ConversationEventsTypedDict(TypedDict):
    event: SSETypes
    r"""Server side events sent when streaming a conversation response."""
    data: ConversationEventsDataTypedDict


class ConversationEvents(BaseModel):
    event: SSETypes
    r"""Server side events sent when streaming a conversation response."""

    data: ConversationEventsData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .classifierjobout import ClassifierJobOut, ClassifierJobOutTypedDict
from .completionjobout import CompletionJobOut, CompletionJobOutTypedDict
from .legacyjobmetadataout import LegacyJobMetadataOut, LegacyJobMetadataOutTypedDict
from mistralai.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


Response1TypedDict = TypeAliasType(
    "Response1TypedDict", Union[ClassifierJobOutTypedDict, CompletionJobOutTypedDict]
)


Response1 = Annotated[
    Union[
        Annotated[ClassifierJobOut, Tag("classifier")],
        Annotated[CompletionJobOut, Tag("completion")],
    ],
    Discriminator(lambda m: get_discriminator(m, "job_type", "job_type")),
]


JobsAPIRoutesFineTuningCreateFineTuningJobResponseTypedDict = TypeAliasType(
    "JobsAPIRoutesFineTuningCreateFineTuningJobResponseTypedDict",
    Union[LegacyJobMetadataOutTypedDict, Response1TypedDict],
)
r"""OK"""


JobsAPIRoutesFineTuningCreateFineTuningJobResponse = TypeAliasType(
    "JobsAPIRoutesFineTuningCreateFineTuningJobResponse",
    Union[LegacyJobMetadataOut, Response1],
)
r"""OK"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .libraryout import LibraryOut, LibraryOutTypedDict
from mistralai.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListLibraryOutTypedDict(TypedDict):
    data: List[LibraryOutTypedDict]


class ListLibraryOut(BaseModel):
    data: List[LibraryOut]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .apiendpoint import APIEndpoint
from .batchrequest import BatchRequest, BatchRequestTypedDict
from mistralai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class BatchJobInTypedDict(TypedDict):
    endpoint: APIEndpoint
    input_files: NotRequired[Nullable[List[str]]]
    r"""The list of input files to be used for batch inference, these files should be `jsonl` files, containing the input data corresponding to the bory request for the batch inference in a \"body\" field. An example of such file is the following: ```json {\"custom_id\": \"0\", \"body\": {\"max_tokens\": 100, \"messages\": [{\"role\": \"user\", \"content\": \"What is the best French cheese?\"}]}} {\"custom_id\": \"1\", \"body\": {\"max_tokens\": 100, \"messages\": [{\"role\": \"user\", \"content\": \"What is the best French wine?\"}]}} ```"""
    requests: NotRequired[Nullable[List[BatchRequestTypedDict]]]
    model: NotRequired[Nullable[str]]
    r"""The model to be used for batch inference."""
    agent_id: NotRequired[Nullable[str]]
    r"""In case you want to use a specific agent from the **deprecated** agents api for batch inference, you can specify the agent ID here."""
    metadata: NotRequired[Nullable[Dict[str, str]]]
    r"""The metadata of your choice to be associated with the batch inference job."""
    timeout_hours: NotRequired[int]
    r"""The timeout in hours for the batch inference job."""


class BatchJobIn(BaseModel):
    endpoint: APIEndpoint

    input_files: OptionalNullable[List[str]] = UNSET
    r"""The list of input files to be used for batch inference, these files should be `jsonl` files, containing the input data corresponding to the bory request for the batch inference in a \"body\" field. An example of such file is the following: ```json {\"custom_id\": \"0\", \"body\": {\"max_tokens\": 100, \"messages\": [{\"role\": \"user\", \"content\": \"What is the best French cheese?\"}]}} {\"custom_id\": \"1\", \"body\": {\"max_tokens\": 100, \"messages\": [{\"role\": \"user\", \"content\": \"What is the best French wine?\"}]}} ```"""

    requests: OptionalNullable[List[BatchRequest]] = UNSET

    model: OptionalNullable[str] = UNSET
    r"""The model to be used for batch inference."""

    agent_id: OptionalNullable[str] = UNSET
    r"""In case you want to use a specific agent from the **deprecated** agents api for batch inference, you can specify the agent ID here."""

    metadata: OptionalNullable[Dict[str, str]] = UNSET
    r"""The metadata of your choice to be associated with the batch inference job."""

    timeout_hours: Optional[int] = 24
    r"""The timeout in hours for the batch inference job."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "input_files",
            "requests",
            "model",
            "agent_id",
            "metadata",
            "timeout_hours",
        ]
        nullable_fields = ["input_files", "requests", "model", "agent_id", "metadata"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from mistralai.types import BaseModel
from mistralai.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AgentsAPIV1AgentsListVersionsRequestTypedDict(TypedDict):
    agent_id: str
    page: NotRequired[int]
    r"""Page number (0-indexed)"""
    page_size: NotRequired[int]
    r"""Number of versions per page"""


class AgentsAPIV1AgentsListVersionsRequest(BaseModel):
    agent_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 0
    r"""Page number (0-indexed)"""

    page_size: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 20
    r"""Number of versions per page"""



from ..utils.response_format import (
    pydantic_model_from_json,
    response_format_from_pydantic_model,
    rec_strict_json_schema,
)
from pydantic import BaseModel, ValidationError

from ...models import ResponseFormat, JSONSchema
from ...types.basemodel import Unset

import unittest


class Student(BaseModel):
    name: str
    age: int


class Explanation(BaseModel):
    explanation: str
    output: str


class MathDemonstration(BaseModel):
    steps: list[Explanation]
    final_answer: str


mathdemo_schema = {
    "$defs": {
        "Explanation": {
            "properties": {
                "explanation": {"title": "Explanation", "type": "string"},
                "output": {"title": "Output", "type": "string"},
            },
            "required": ["explanation", "output"],
            "title": "Explanation",
            "type": "object",
        }
    },
    "properties": {
        "steps": {
            "items": {"$ref": "#/$defs/Explanation"},
            "title": "Steps",
            "type": "array",
        },
        "final_answer": {"title": "Final Answer", "type": "string"},
    },
    "required": ["steps", "final_answer"],
    "title": "MathDemonstration",
    "type": "object",
}

mathdemo_strict_schema = mathdemo_schema.copy()
mathdemo_strict_schema["$defs"]["Explanation"]["additionalProperties"] = False # type: ignore
mathdemo_strict_schema["additionalProperties"] = False

mathdemo_response_format = ResponseFormat(
    type="json_schema",
    json_schema=JSONSchema(
        name="MathDemonstration",
        schema_definition=mathdemo_strict_schema,
        description=Unset(),
        strict=True,
    ),
)


class TestResponseFormat(unittest.TestCase):
    def test_pydantic_model_from_json(self):
        missing_json_data = {"name": "Jean Dupont"}
        good_json_data = {"name": "Jean Dupont", "age": 25}
        extra_json_data = {
            "name": "Jean Dupont",
            "age": 25,
            "extra_field": "extra_value",
        }
        complex_json_data = {
            "final_answer": "x = -4",
            "steps": [
                {
                    "explanation": "Start with the given equation.",
                    "output": "8x + 7 = -23",
                },
                {
                    "explanation": "Subtract 7 from both sides to isolate the term with x.",
                    "output": "8x = -23 - 7",
                },
                {
                    "explanation": "Simplify the right side of the equation.",
                    "output": "8x = -30",
                },
                {
                    "explanation": "Divide both sides by 8 to solve for x.",
                    "output": "x = -30 / 8",
                },
                {
                    "explanation": "Simplify the fraction to get the final answer.",
                    "output": "x = -4",
                },
            ],
        }

        self.assertEqual(
            pydantic_model_from_json(good_json_data, Student),
            Student(name="Jean Dupont", age=25),
        )
        self.assertEqual(
            pydantic_model_from_json(extra_json_data, Student),
            Student(name="Jean Dupont", age=25),
        )
        self.assertEqual(
            pydantic_model_from_json(complex_json_data, MathDemonstration),
            MathDemonstration(
                steps=[
                    Explanation(
                        explanation="Start with the given equation.",
                        output="8x + 7 = -23",
                    ),
                    Explanation(
                        explanation="Subtract 7 from both sides to isolate the term with x.",
                        output="8x = -23 - 7",
                    ),
                    Explanation(
                        explanation="Simplify the right side of the equation.",
                        output="8x = -30",
                    ),
                    Explanation(
                        explanation="Divide both sides by 8 to solve for x.",
                        output="x = -30 / 8",
                    ),
                    Explanation(
                        explanation="Simplify the fraction to get the final answer.",
                        output="x = -4",
                    ),
                ],
                final_answer="x = -4",
            ),
        )

        # Check it raises a validation error
        with self.assertRaises(ValidationError):
            pydantic_model_from_json(missing_json_data, Student)  # type: ignore

    def test_response_format_from_pydantic_model(self):
        self.assertEqual(
            response_format_from_pydantic_model(MathDemonstration),
            mathdemo_response_format,
        )

    def test_rec_strict_json_schema(self):
        invalid_schema = mathdemo_schema | {"wrong_value": 1}
        self.assertEqual(
            rec_strict_json_schema(mathdemo_schema), mathdemo_strict_schema
        )

        with self.assertRaises(ValueError):
            rec_strict_json_schema(invalid_schema)


if __name__ == "__main__":
    unittest.main()



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from mistralai.types import BaseModel
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


WebSearchPremiumToolType = Literal["web_search_premium",]


class WebSearchPremiumToolTypedDict(TypedDict):
    type: NotRequired[WebSearchPremiumToolType]


class WebSearchPremiumTool(BaseModel):
    type: Optional[WebSearchPremiumToolType] = "web_search_premium"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .audiochunk import AudioChunk, AudioChunkTypedDict
from .documenturlchunk import DocumentURLChunk, DocumentURLChunkTypedDict
from .filechunk import FileChunk, FileChunkTypedDict
from .imageurlchunk import ImageURLChunk, ImageURLChunkTypedDict
from .referencechunk import ReferenceChunk, ReferenceChunkTypedDict
from .textchunk import TextChunk, TextChunkTypedDict
from .thinkchunk import ThinkChunk, ThinkChunkTypedDict
from mistralai.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


ContentChunkTypedDict = TypeAliasType(
    "ContentChunkTypedDict",
    Union[
        TextChunkTypedDict,
        ImageURLChunkTypedDict,
        ReferenceChunkTypedDict,
        FileChunkTypedDict,
        AudioChunkTypedDict,
        DocumentURLChunkTypedDict,
        ThinkChunkTypedDict,
    ],
)


ContentChunk = Annotated[
    Union[
        Annotated[ImageURLChunk, Tag("image_url")],
        Annotated[DocumentURLChunk, Tag("document_url")],
        Annotated[TextChunk, Tag("text")],
        Annotated[ReferenceChunk, Tag("reference")],
        Annotated[FileChunk, Tag("file")],
        Annotated[ThinkChunk, Tag("thinking")],
        Annotated[AudioChunk, Tag("input_audio")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from mistralai.types import BaseModel
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


DocumentLibraryToolType = Literal["document_library",]


class DocumentLibraryToolTypedDict(TypedDict):
    library_ids: List[str]
    r"""Ids of the library in which to search."""
    type: NotRequired[DocumentLibraryToolType]


class DocumentLibraryTool(BaseModel):
    library_ids: List[str]
    r"""Ids of the library in which to search."""

    type: Optional[DocumentLibraryToolType] = "document_library"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .completionargsstop import CompletionArgsStop, CompletionArgsStopTypedDict
from .prediction import Prediction, PredictionTypedDict
from .responseformat import ResponseFormat, ResponseFormatTypedDict
from .toolchoiceenum import ToolChoiceEnum
from mistralai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class CompletionArgsTypedDict(TypedDict):
    r"""White-listed arguments from the completion API"""

    stop: NotRequired[Nullable[CompletionArgsStopTypedDict]]
    presence_penalty: NotRequired[Nullable[float]]
    frequency_penalty: NotRequired[Nullable[float]]
    temperature: NotRequired[Nullable[float]]
    top_p: NotRequired[Nullable[float]]
    max_tokens: NotRequired[Nullable[int]]
    random_seed: NotRequired[Nullable[int]]
    prediction: NotRequired[Nullable[PredictionTypedDict]]
    response_format: NotRequired[Nullable[ResponseFormatTypedDict]]
    tool_choice: NotRequired[ToolChoiceEnum]


class CompletionArgs(BaseModel):
    r"""White-listed arguments from the completion API"""

    stop: OptionalNullable[CompletionArgsStop] = UNSET

    presence_penalty: OptionalNullable[float] = UNSET

    frequency_penalty: OptionalNullable[float] = UNSET

    temperature: OptionalNullable[float] = UNSET

    top_p: OptionalNullable[float] = UNSET

    max_tokens: OptionalNullable[int] = UNSET

    random_seed: OptionalNullable[int] = UNSET

    prediction: OptionalNullable[Prediction] = UNSET

    response_format: OptionalNullable[ResponseFormat] = UNSET

    tool_choice: Optional[ToolChoiceEnum] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "stop",
            "presence_penalty",
            "frequency_penalty",
            "temperature",
            "top_p",
            "max_tokens",
            "random_seed",
            "prediction",
            "response_format",
            "tool_choice",
        ]
        nullable_fields = [
            "stop",
            "presence_penalty",
            "frequency_penalty",
            "temperature",
            "top_p",
            "max_tokens",
            "random_seed",
            "prediction",
            "response_format",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .ftmodelcapabilitiesout import (
    FTModelCapabilitiesOut,
    FTModelCapabilitiesOutTypedDict,
)
from mistralai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


CompletionFTModelOutObject = Literal["model",]


ModelType = Literal["completion",]


class CompletionFTModelOutTypedDict(TypedDict):
    id: str
    created: int
    owned_by: str
    workspace_id: str
    root: str
    root_version: str
    archived: bool
    capabilities: FTModelCapabilitiesOutTypedDict
    job: str
    object: NotRequired[CompletionFTModelOutObject]
    name: NotRequired[Nullable[str]]
    description: NotRequired[Nullable[str]]
    max_context_length: NotRequired[int]
    aliases: NotRequired[List[str]]
    model_type: NotRequired[ModelType]


class CompletionFTModelOut(BaseModel):
    id: str

    created: int

    owned_by: str

    workspace_id: str

    root: str

    root_version: str

    archived: bool

    capabilities: FTModelCapabilitiesOut

    job: str

    object: Optional[CompletionFTModelOutObject] = "model"

    name: OptionalNullable[str] = UNSET

    description: OptionalNullable[str] = UNSET

    max_context_length: Optional[int] = 32768

    aliases: Optional[List[str]] = None

    model_type: Optional[ModelType] = "completion"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "object",
            "name",
            "description",
            "max_context_length",
            "aliases",
            "model_type",
        ]
        nullable_fields = ["name", "description"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .filepurpose import FilePurpose
from .sampletype import SampleType
from .source import Source
from mistralai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing_extensions import Annotated, NotRequired, TypedDict


class RetrieveFileOutTypedDict(TypedDict):
    id: str
    r"""The unique identifier of the file."""
    object: str
    r"""The object type, which is always \"file\"."""
    size_bytes: int
    r"""The size of the file, in bytes."""
    created_at: int
    r"""The UNIX timestamp (in seconds) of the event."""
    filename: str
    r"""The name of the uploaded file."""
    purpose: FilePurpose
    sample_type: SampleType
    source: Source
    deleted: bool
    num_lines: NotRequired[Nullable[int]]
    mimetype: NotRequired[Nullable[str]]
    signature: NotRequired[Nullable[str]]


class RetrieveFileOut(BaseModel):
    id: str
    r"""The unique identifier of the file."""

    object: str
    r"""The object type, which is always \"file\"."""

    size_bytes: Annotated[int, pydantic.Field(alias="bytes")]
    r"""The size of the file, in bytes."""

    created_at: int
    r"""The UNIX timestamp (in seconds) of the event."""

    filename: str
    r"""The name of the uploaded file."""

    purpose: FilePurpose

    sample_type: SampleType

    source: Source

    deleted: bool

    num_lines: OptionalNullable[int] = UNSET

    mimetype: OptionalNullable[str] = UNSET

    signature: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["num_lines", "mimetype", "signature"]
        nullable_fields = ["num_lines", "mimetype", "signature"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .inputs import Inputs, InputsTypedDict
from mistralai.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class ChatClassificationRequestTypedDict(TypedDict):
    model: str
    inputs: InputsTypedDict
    r"""Chat to classify"""


class ChatClassificationRequest(BaseModel):
    model: str

    inputs: Annotated[Inputs, pydantic.Field(alias="input")]
    r"""Chat to classify"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .outputcontentchunks import OutputContentChunks, OutputContentChunksTypedDict
from datetime import datetime
from mistralai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Literal, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


MessageOutputEventType = Literal["message.output.delta",]


MessageOutputEventRole = Literal["assistant",]


MessageOutputEventContentTypedDict = TypeAliasType(
    "MessageOutputEventContentTypedDict", Union[str, OutputContentChunksTypedDict]
)


MessageOutputEventContent = TypeAliasType(
    "MessageOutputEventContent", Union[str, OutputContentChunks]
)


class MessageOutputEventTypedDict(TypedDict):
    id: str
    content: MessageOutputEventContentTypedDict
    type: NotRequired[MessageOutputEventType]
    created_at: NotRequired[datetime]
    output_index: NotRequired[int]
    content_index: NotRequired[int]
    model: NotRequired[Nullable[str]]
    agent_id: NotRequired[Nullable[str]]
    role: NotRequired[MessageOutputEventRole]


class MessageOutputEvent(BaseModel):
    id: str

    content: MessageOutputEventContent

    type: Optional[MessageOutputEventType] = "message.output.delta"

    created_at: Optional[datetime] = None

    output_index: Optional[int] = 0

    content_index: Optional[int] = 0

    model: OptionalNullable[str] = UNSET

    agent_id: OptionalNullable[str] = UNSET

    role: Optional[MessageOutputEventRole] = "assistant"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "type",
            "created_at",
            "output_index",
            "content_index",
            "model",
            "agent_id",
            "role",
        ]
        nullable_fields = ["model", "agent_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .classifiertargetin import ClassifierTargetIn, ClassifierTargetInTypedDict
from .classifiertrainingparametersin import (
    ClassifierTrainingParametersIn,
    ClassifierTrainingParametersInTypedDict,
)
from .completiontrainingparametersin import (
    CompletionTrainingParametersIn,
    CompletionTrainingParametersInTypedDict,
)
from .finetuneablemodeltype import FineTuneableModelType
from .githubrepositoryin import GithubRepositoryIn, GithubRepositoryInTypedDict
from .trainingfile import TrainingFile, TrainingFileTypedDict
from .wandbintegration import WandbIntegration, WandbIntegrationTypedDict
from mistralai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


JobInIntegrationsTypedDict = WandbIntegrationTypedDict


JobInIntegrations = WandbIntegration


HyperparametersTypedDict = TypeAliasType(
    "HyperparametersTypedDict",
    Union[
        ClassifierTrainingParametersInTypedDict, CompletionTrainingParametersInTypedDict
    ],
)


Hyperparameters = TypeAliasType(
    "Hyperparameters",
    Union[ClassifierTrainingParametersIn, CompletionTrainingParametersIn],
)


JobInRepositoriesTypedDict = GithubRepositoryInTypedDict


JobInRepositories = GithubRepositoryIn


class JobInTypedDict(TypedDict):
    model: str
    r"""The name of the model to fine-tune."""
    hyperparameters: HyperparametersTypedDict
    training_files: NotRequired[List[TrainingFileTypedDict]]
    validation_files: NotRequired[Nullable[List[str]]]
    r"""A list containing the IDs of uploaded files that contain validation data. If you provide these files, the data is used to generate validation metrics periodically during fine-tuning. These metrics can be viewed in `checkpoints` when getting the status of a running fine-tuning job. The same data should not be present in both train and validation files."""
    suffix: NotRequired[Nullable[str]]
    r"""A string that will be added to your fine-tuning model name. For example, a suffix of \"my-great-model\" would produce a model name like `ft:open-mistral-7b:my-great-model:xxx...`"""
    integrations: NotRequired[Nullable[List[JobInIntegrationsTypedDict]]]
    r"""A list of integrations to enable for your fine-tuning job."""
    auto_start: NotRequired[bool]
    r"""This field will be required in a future release."""
    invalid_sample_skip_percentage: NotRequired[float]
    job_type: NotRequired[Nullable[FineTuneableModelType]]
    repositories: NotRequired[Nullable[List[JobInRepositoriesTypedDict]]]
    classifier_targets: NotRequired[Nullable[List[ClassifierTargetInTypedDict]]]


class JobIn(BaseModel):
    model: str
    r"""The name of the model to fine-tune."""

    hyperparameters: Hyperparameters

    training_files: Optional[List[TrainingFile]] = None

    validation_files: OptionalNullable[List[str]] = UNSET
    r"""A list containing the IDs of uploaded files that contain validation data. If you provide these files, the data is used to generate validation metrics periodically during fine-tuning. These metrics can be viewed in `checkpoints` when getting the status of a running fine-tuning job. The same data should not be present in both train and validation files."""

    suffix: OptionalNullable[str] = UNSET
    r"""A string that will be added to your fine-tuning model name. For example, a suffix of \"my-great-model\" would produce a model name like `ft:open-mistral-7b:my-great-model:xxx...`"""

    integrations: OptionalNullable[List[JobInIntegrations]] = UNSET
    r"""A list of integrations to enable for your fine-tuning job."""

    auto_start: Optional[bool] = None
    r"""This field will be required in a future release."""

    invalid_sample_skip_percentage: Optional[float] = 0

    job_type: OptionalNullable[FineTuneableModelType] = UNSET

    repositories: OptionalNullable[List[JobInRepositories]] = UNSET

    classifier_targets: OptionalNullable[List[ClassifierTargetIn]] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "training_files",
            "validation_files",
            "suffix",
            "integrations",
            "auto_start",
            "invalid_sample_skip_percentage",
            "job_type",
            "repositories",
            "classifier_targets",
        ]
        nullable_fields = [
            "validation_files",
            "suffix",
            "integrations",
            "job_type",
            "repositories",
            "classifier_targets",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

import json
import weakref
from typing import Any, Optional, Union, cast

import google.auth
import google.auth.credentials
import google.auth.transport
import google.auth.transport.requests
import httpx

from mistralai_gcp import models
from mistralai_gcp._hooks import BeforeRequestHook, SDKHooks
from mistralai_gcp.chat import Chat
from mistralai_gcp.fim import Fim
from mistralai_gcp.types import UNSET, OptionalNullable

from .basesdk import BaseSDK
from .httpclient import AsyncHttpClient, ClientOwner, HttpClient, close_clients
from .sdkconfiguration import SDKConfiguration
from .utils.logger import Logger, get_default_logger
from .utils.retries import RetryConfig

LEGACY_MODEL_ID_FORMAT = {
    "codestral-2405": "codestral@2405",
    "mistral-large-2407": "mistral-large@2407",
    "mistral-nemo-2407": "mistral-nemo@2407",
}


def get_model_info(model: str) -> tuple[str, str]:
    # if the model requiers the legacy fomat, use it, else do nothing.
    if model in LEGACY_MODEL_ID_FORMAT:
        return "-".join(model.split("-")[:-1]), LEGACY_MODEL_ID_FORMAT[model]
    return model, model


class MistralGoogleCloud(BaseSDK):
    r"""Mistral AI API: Our Chat Completion and Embeddings APIs specification. Create your account on [La Plateforme](https://console.mistral.ai) to get access and read the [docs](https://docs.mistral.ai) to learn how to use it."""

    chat: Chat
    r"""Chat Completion API."""
    fim: Fim
    r"""Fill-in-the-middle API."""

    def __init__(
        self,
        region: str = "europe-west4",
        project_id: Optional[str] = None,
        access_token: Optional[str] = None,
        client: Optional[HttpClient] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None,
    ) -> None:
        r"""Instantiates the SDK configuring it with the provided parameters.

        :param api_key: The api_key required for authentication
        :param server: The server by name to use for all methods
        :param server_url: The server URL to use for all methods
        :param url_params: Parameters to optionally template the server URL with
        :param client: The HTTP client to use for all synchronous methods
        :param async_client: The Async HTTP client to use for all asynchronous methods
        :param retry_config: The retry configuration to use for all supported methods
        :param timeout_ms: Optional request timeout applied to each operation in milliseconds
        """

        credentials = None
        if not access_token:
            credentials, loaded_project_id = google.auth.default(
                scopes=["https://www.googleapis.com/auth/cloud-platform"],
            )

            # default will already raise a google.auth.exceptions.DefaultCredentialsError if no credentials are found
            assert isinstance(
                credentials, google.auth.credentials.Credentials
            ), "credentials must be an instance of google.auth.credentials.Credentials"

            credentials.refresh(google.auth.transport.requests.Request())
            project_id = project_id or loaded_project_id

        if project_id is None:
            raise ValueError("project_id must be provided")

        def auth_token() -> str:
            if access_token:
                return access_token

            assert credentials is not None, "credentials must be initialized"
            credentials.refresh(google.auth.transport.requests.Request())
            token = credentials.token
            if not token:
                raise Exception("Failed to get token from credentials")
            return token

        client_supplied = True
        if client is None:
            client = httpx.Client()
            client_supplied = False

        assert issubclass(
            type(client), HttpClient
        ), "The provided client must implement the HttpClient protocol."

        async_client_supplied = True
        if async_client is None:
            async_client = httpx.AsyncClient()
            async_client_supplied = False

        if debug_logger is None:
            debug_logger = get_default_logger()

        assert issubclass(
            type(async_client), AsyncHttpClient
        ), "The provided async_client must implement the AsyncHttpClient protocol."

        security: Any = None
        if callable(auth_token):
            security = lambda: models.Security(  # pylint: disable=unnecessary-lambda-assignment
                api_key=auth_token()
            )
        else:
            security = models.Security(api_key=auth_token)

        BaseSDK.__init__(
            self,
            SDKConfiguration(
                client=client,
                client_supplied=client_supplied,
                async_client=async_client,
                async_client_supplied=async_client_supplied,
                security=security,
                server_url=f"https://{region}-aiplatform.googleapis.com",
                server=None,
                retry_config=retry_config,
                timeout_ms=timeout_ms,
                debug_logger=debug_logger,
            ),
        )

        hooks = SDKHooks()
        hook = GoogleCloudBeforeRequestHook(region, project_id)
        hooks.register_before_request_hook(hook)
        current_server_url, *_ = self.sdk_configuration.get_server_details()
        server_url, self.sdk_configuration.client = hooks.sdk_init(
            current_server_url, client
        )
        if current_server_url != server_url:
            self.sdk_configuration.server_url = server_url

        # pylint: disable=protected-access
        self.sdk_configuration.__dict__["_hooks"] = hooks

        weakref.finalize(
            self,
            close_clients,
            cast(ClientOwner, self.sdk_configuration),
            self.sdk_configuration.client,
            self.sdk_configuration.client_supplied,
            self.sdk_configuration.async_client,
            self.sdk_configuration.async_client_supplied,
        )

        self._init_sdks()

    def _init_sdks(self):
        self.chat = Chat(self.sdk_configuration)
        self.fim = Fim(self.sdk_configuration)

    def __enter__(self):
        return self

    async def __aenter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.client is not None
            and not self.sdk_configuration.client_supplied
        ):
            self.sdk_configuration.client.close()
        self.sdk_configuration.client = None

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.async_client is not None
            and not self.sdk_configuration.async_client_supplied
        ):
            await self.sdk_configuration.async_client.aclose()
        self.sdk_configuration.async_client = None


class GoogleCloudBeforeRequestHook(BeforeRequestHook):
    def __init__(self, region: str, project_id: str):
        self.region = region
        self.project_id = project_id

    def before_request(
        self, hook_ctx, request: httpx.Request
    ) -> Union[httpx.Request, Exception]:
        # The goal of this function is to template in the region, project and model into the URL path
        # We do this here so that the API remains more user-friendly
        model_id = None
        new_content = None
        if request.content:
            parsed = json.loads(request.content.decode("utf-8"))
            model_raw = parsed.get("model")
            model_name, model_id = get_model_info(model_raw)
            parsed["model"] = model_name
            new_content = json.dumps(parsed).encode("utf-8")

        if model_id == "":
            raise ValueError("model must be provided")

        stream = "streamRawPredict" in request.url.path
        specifier = "streamRawPredict" if stream else "rawPredict"
        url = f"/v1/projects/{self.project_id}/locations/{self.region}/publishers/mistralai/models/{model_id}:{specifier}"

        headers = dict(request.headers)
        # Delete content-length header as it will need to be recalculated
        headers.pop("content-length", None)

        next_request = httpx.Request(
            method=request.method,
            url=request.url.copy_with(path=url),
            headers=headers,
            content=new_content,
            stream=None,
        )

        return next_request



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .checkpointout import CheckpointOut, CheckpointOutTypedDict
from .completiontrainingparameters import (
    CompletionTrainingParameters,
    CompletionTrainingParametersTypedDict,
)
from .eventout import EventOut, EventOutTypedDict
from .githubrepositoryout import GithubRepositoryOut, GithubRepositoryOutTypedDict
from .jobmetadataout import JobMetadataOut, JobMetadataOutTypedDict
from .wandbintegrationout import WandbIntegrationOut, WandbIntegrationOutTypedDict
from mistralai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


CompletionDetailedJobOutStatus = Literal[
    "QUEUED",
    "STARTED",
    "VALIDATING",
    "VALIDATED",
    "RUNNING",
    "FAILED_VALIDATION",
    "FAILED",
    "SUCCESS",
    "CANCELLED",
    "CANCELLATION_REQUESTED",
]


CompletionDetailedJobOutObject = Literal["job",]


CompletionDetailedJobOutIntegrationsTypedDict = WandbIntegrationOutTypedDict


CompletionDetailedJobOutIntegrations = WandbIntegrationOut


CompletionDetailedJobOutJobType = Literal["completion",]


CompletionDetailedJobOutRepositoriesTypedDict = GithubRepositoryOutTypedDict


CompletionDetailedJobOutRepositories = GithubRepositoryOut


class CompletionDetailedJobOutTypedDict(TypedDict):
    id: str
    auto_start: bool
    model: str
    r"""The name of the model to fine-tune."""
    status: CompletionDetailedJobOutStatus
    created_at: int
    modified_at: int
    training_files: List[str]
    hyperparameters: CompletionTrainingParametersTypedDict
    validation_files: NotRequired[Nullable[List[str]]]
    object: NotRequired[CompletionDetailedJobOutObject]
    fine_tuned_model: NotRequired[Nullable[str]]
    suffix: NotRequired[Nullable[str]]
    integrations: NotRequired[
        Nullable[List[CompletionDetailedJobOutIntegrationsTypedDict]]
    ]
    trained_tokens: NotRequired[Nullable[int]]
    metadata: NotRequired[Nullable[JobMetadataOutTypedDict]]
    job_type: NotRequired[CompletionDetailedJobOutJobType]
    repositories: NotRequired[List[CompletionDetailedJobOutRepositoriesTypedDict]]
    events: NotRequired[List[EventOutTypedDict]]
    r"""Event items are created every time the status of a fine-tuning job changes. The timestamped list of all events is accessible here."""
    checkpoints: NotRequired[List[CheckpointOutTypedDict]]


class CompletionDetailedJobOut(BaseModel):
    id: str

    auto_start: bool

    model: str
    r"""The name of the model to fine-tune."""

    status: CompletionDetailedJobOutStatus

    created_at: int

    modified_at: int

    training_files: List[str]

    hyperparameters: CompletionTrainingParameters

    validation_files: OptionalNullable[List[str]] = UNSET

    object: Optional[CompletionDetailedJobOutObject] = "job"

    fine_tuned_model: OptionalNullable[str] = UNSET

    suffix: OptionalNullable[str] = UNSET

    integrations: OptionalNullable[List[CompletionDetailedJobOutIntegrations]] = UNSET

    trained_tokens: OptionalNullable[int] = UNSET

    metadata: OptionalNullable[JobMetadataOut] = UNSET

    job_type: Optional[CompletionDetailedJobOutJobType] = "completion"

    repositories: Optional[List[CompletionDetailedJobOutRepositories]] = None

    events: Optional[List[EventOut]] = None
    r"""Event items are created every time the status of a fine-tuning job changes. The timestamped list of all events is accessible here."""

    checkpoints: Optional[List[CheckpointOut]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "validation_files",
            "object",
            "fine_tuned_model",
            "suffix",
            "integrations",
            "trained_tokens",
            "metadata",
            "job_type",
            "repositories",
            "events",
            "checkpoints",
        ]
        nullable_fields = [
            "validation_files",
            "fine_tuned_model",
            "suffix",
            "integrations",
            "trained_tokens",
            "metadata",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .agentupdaterequest import AgentUpdateRequest, AgentUpdateRequestTypedDict
from mistralai.types import BaseModel
from mistralai.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class AgentsAPIV1AgentsUpdateRequestTypedDict(TypedDict):
    agent_id: str
    agent_update_request: AgentUpdateRequestTypedDict


class AgentsAPIV1AgentsUpdateRequest(BaseModel):
    agent_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    agent_update_request: Annotated[
        AgentUpdateRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from mistralai.types import BaseModel
from typing_extensions import TypedDict


class FileSignedURLTypedDict(TypedDict):
    url: str


class FileSignedURL(BaseModel):
    url: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from mistralai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from mistralai.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class LibrariesDocumentsListV1RequestTypedDict(TypedDict):
    library_id: str
    search: NotRequired[Nullable[str]]
    page_size: NotRequired[int]
    page: NotRequired[int]
    filters_attributes: NotRequired[Nullable[str]]
    sort_by: NotRequired[str]
    sort_order: NotRequired[str]


class LibrariesDocumentsListV1Request(BaseModel):
    library_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    search: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    page_size: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 100

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 0

    filters_attributes: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    sort_by: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = "created_at"

    sort_order: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = "desc"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "search",
            "page_size",
            "page",
            "filters_attributes",
            "sort_by",
            "sort_order",
        ]
        nullable_fields = ["search", "filters_attributes"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .completiontrainingparameters import (
    CompletionTrainingParameters,
    CompletionTrainingParametersTypedDict,
)
from .githubrepositoryout import GithubRepositoryOut, GithubRepositoryOutTypedDict
from .jobmetadataout import JobMetadataOut, JobMetadataOutTypedDict
from .wandbintegrationout import WandbIntegrationOut, WandbIntegrationOutTypedDict
from mistralai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


Status = Literal[
    "QUEUED",
    "STARTED",
    "VALIDATING",
    "VALIDATED",
    "RUNNING",
    "FAILED_VALIDATION",
    "FAILED",
    "SUCCESS",
    "CANCELLED",
    "CANCELLATION_REQUESTED",
]
r"""The current status of the fine-tuning job."""


CompletionJobOutObject = Literal["job",]
r"""The object type of the fine-tuning job."""


IntegrationsTypedDict = WandbIntegrationOutTypedDict


Integrations = WandbIntegrationOut


JobType = Literal["completion",]
r"""The type of job (`FT` for fine-tuning)."""


RepositoriesTypedDict = GithubRepositoryOutTypedDict


Repositories = GithubRepositoryOut


class CompletionJobOutTypedDict(TypedDict):
    id: str
    r"""The ID of the job."""
    auto_start: bool
    model: str
    r"""The name of the model to fine-tune."""
    status: Status
    r"""The current status of the fine-tuning job."""
    created_at: int
    r"""The UNIX timestamp (in seconds) for when the fine-tuning job was created."""
    modified_at: int
    r"""The UNIX timestamp (in seconds) for when the fine-tuning job was last modified."""
    training_files: List[str]
    r"""A list containing the IDs of uploaded files that contain training data."""
    hyperparameters: CompletionTrainingParametersTypedDict
    validation_files: NotRequired[Nullable[List[str]]]
    r"""A list containing the IDs of uploaded files that contain validation data."""
    object: NotRequired[CompletionJobOutObject]
    r"""The object type of the fine-tuning job."""
    fine_tuned_model: NotRequired[Nullable[str]]
    r"""The name of the fine-tuned model that is being created. The value will be `null` if the fine-tuning job is still running."""
    suffix: NotRequired[Nullable[str]]
    r"""Optional text/code that adds more context for the model. When given a `prompt` and a `suffix` the model will fill what is between them. When `suffix` is not provided, the model will simply execute completion starting with `prompt`."""
    integrations: NotRequired[Nullable[List[IntegrationsTypedDict]]]
    r"""A list of integrations enabled for your fine-tuning job."""
    trained_tokens: NotRequired[Nullable[int]]
    r"""Total number of tokens trained."""
    metadata: NotRequired[Nullable[JobMetadataOutTypedDict]]
    job_type: NotRequired[JobType]
    r"""The type of job (`FT` for fine-tuning)."""
    repositories: NotRequired[List[RepositoriesTypedDict]]


class CompletionJobOut(BaseModel):
    id: str
    r"""The ID of the job."""

    auto_start: bool

    model: str
    r"""The name of the model to fine-tune."""

    status: Status
    r"""The current status of the fine-tuning job."""

    created_at: int
    r"""The UNIX timestamp (in seconds) for when the fine-tuning job was created."""

    modified_at: int
    r"""The UNIX timestamp (in seconds) for when the fine-tuning job was last modified."""

    training_files: List[str]
    r"""A list containing the IDs of uploaded files that contain training data."""

    hyperparameters: CompletionTrainingParameters

    validation_files: OptionalNullable[List[str]] = UNSET
    r"""A list containing the IDs of uploaded files that contain validation data."""

    object: Optional[CompletionJobOutObject] = "job"
    r"""The object type of the fine-tuning job."""

    fine_tuned_model: OptionalNullable[str] = UNSET
    r"""The name of the fine-tuned model that is being created. The value will be `null` if the fine-tuning job is still running."""

    suffix: OptionalNullable[str] = UNSET
    r"""Optional text/code that adds more context for the model. When given a `prompt` and a `suffix` the model will fill what is between them. When `suffix` is not provided, the model will simply execute completion starting with `prompt`."""

    integrations: OptionalNullable[List[Integrations]] = UNSET
    r"""A list of integrations enabled for your fine-tuning job."""

    trained_tokens: OptionalNullable[int] = UNSET
    r"""Total number of tokens trained."""

    metadata: OptionalNullable[JobMetadataOut] = UNSET

    job_type: Optional[JobType] = "completion"
    r"""The type of job (`FT` for fine-tuning)."""

    repositories: Optional[List[Repositories]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "validation_files",
            "object",
            "fine_tuned_model",
            "suffix",
            "integrations",
            "trained_tokens",
            "metadata",
            "job_type",
            "repositories",
        ]
        nullable_fields = [
            "validation_files",
            "fine_tuned_model",
            "suffix",
            "integrations",
            "trained_tokens",
            "metadata",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .functioncall import FunctionCall, FunctionCallTypedDict
from .tooltypes import ToolTypes
from mistralai.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ToolCallTypedDict(TypedDict):
    function: FunctionCallTypedDict
    id: NotRequired[str]
    type: NotRequired[ToolTypes]
    index: NotRequired[int]


class ToolCall(BaseModel):
    function: FunctionCall

    id: Optional[str] = "null"

    type: Optional[ToolTypes] = None

    index: Optional[int] = 0
